/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                   */
/*        MADE - Macintosh Application Development Essentials        */
/*        ---------------------------------------------------        */
/*           (c) Sig Software, http://www.sigsoftware.com/           */
/*                                                                   */
/* These files can only be used for experimental purposes. To obtain */
/* fully commented code, source code for the functions in Essential  */
/*   Extras.h and permission for usage in final projects, you must   */
/*    purchase a license. See documentation for more information.    */
/*                                                                   */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                   */
/*  Essential Events.c                                               */
/*  ------------------                                               */
/*                                                                   */
/*  Event and AppleEvent handling.                                   */
/*                                                                   */
/*  Version 1.0.0 - 10th November 1996                               */
/*  Version 1.1.0 - 29th January 1998 - Modeless dialog support      */
/*  Version 1.2.0 - 20th November 1998 - Avoids C/C++ warnings       */
/*                                       Open App event support      */
/*                                       Suspend/resume events       */
/*  Version 1.4.0 - 26th May 1999 - Rearranged, threads, appInFront  */
/*  Version 1.5.0 - 14th December 1999 - Toolbar, Dialog osEvt, idle */
/*                                       Zooming, Display notice     */
/*  Version 1.6.0 - 22nd March 2000 - Added support for Carbon API   */
/*                                                                   */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "Essential Headers.h"
#include "Essential Prototypes.h"
#include "Essential Extras.h"

WindowPtr	frontWindow=0;
WindowPtr	realFrontWindow;
WindowPtr	toolBarWindow=0;
short		toolBarHeight=0;
Boolean		appInFront=true;
void		SetWindowStandardRect(WindowPtr window, short zoomedWidth, short zoomedHeight);

#if Use_AppleEvents

#include <AppleEvents.h>

#if Handle_Open_Application_Event
pascal OSErr		EventOpenAppHandler(const AppleEvent*, AppleEvent*, unsigned long);
AEEventHandlerUPP	EventOpenAppUPP;
#endif

#if Handle_Open_Documents_Event
pascal OSErr		EventOpenDocHandler(const AppleEvent*, AppleEvent*, unsigned long);
AEEventHandlerUPP	EventOpenDocUPP;
#endif

#if Handle_Print_Documents_Event
pascal OSErr		EventPrintDocHandler(const AppleEvent*, AppleEvent*, unsigned long);
AEEventHandlerUPP	EventPrintDocUPP;
#endif

#if Handle_Quit_Application_Event
pascal OSErr		EventQuitAppHandler(const AppleEvent*, AppleEvent*, unsigned long);
AEEventHandlerUPP	EventQuitAppUPP;
#endif

#if Handle_Display_Notice_Event
pascal OSErr		EventDisplayNoticeHandler(const AppleEvent*, AppleEvent*, unsigned long);
AEEventHandlerUPP	EventDisplayNoticeUPP;
#endif

#endif

void MainEventLoop()
{
	EventRecord		theEvent;
	Point			localPoint;
#if Using_Carbon_API
	Cursor			arrowCursor;
#endif
	
#if Add_Threads_Support
	WaitNextEvent(everyEvent, &theEvent,
		ThreadsBusy() ? Event_Sleep_Time_If_Threads_Busy : Event_Sleep_Time, 0);
#else
	WaitNextEvent(everyEvent, &theEvent, Event_Sleep_Time, 0);
#endif
	
	if (frontWindow) {
		localPoint=theEvent.where;
		GlobalToLocal(&localPoint);
		MySetTheCursor(localPoint);
	} else {
#if Using_Carbon_API
		SetCursor(GetQDGlobalsArrow(&arrowCursor));
#else
		SetCursor(&qd.arrow);
#endif
	}

	ProcessEvent(&theEvent);
}

void ProcessEvent(EventRecord* theEvent)
{
	char		keyCharacter;
	short		mousePart, zoomedWidth, zoomedHeight;
	WindowPtr	mouseWindow;
	GrafPtr		oldPort;
	DialogPtr	theDialog;
	Rect		growLimits;
	long		growResult;
	Point		localPoint;
	Error		error=0;
#if Using_Carbon_API
	BitMap		screenBits;
#endif
	
	mousePart=FindWindow(theEvent->where, &mouseWindow);
	
	if (mousePart==inContent) {
		localPoint=theEvent->where;
		
		GetPort(&oldPort);
#if Using_Carbon_API
		SetPort(GetWindowPort(mouseWindow));
#else
		SetPort(mouseWindow);
#endif
		GlobalToLocal(&localPoint);
		SetPort(oldPort);
	}
	
#if Support_Modeless_Dialogs

	if (IsDialogEvent(theEvent)) {
		switch (theEvent->what) {
			case mouseDown:
#if Using_Carbon_API
				theDialog=GetDialogFromWindow(mouseWindow);
#else
				theDialog=mouseWindow;
#endif
				break;
			
			case activateEvt:
				if (theEvent->modifiers&activeFlag) {
					frontWindow=(WindowPtr)theEvent->message;
#if Using_Carbon_API
					SetPort(GetWindowPort(frontWindow));
#else
					SetPort(frontWindow);
#endif
				} else
					frontWindow=0;
			case updateEvt: 
#if Using_Carbon_API
				theDialog=GetDialogFromWindow((WindowPtr)theEvent->message);
#else
				theDialog=(WindowPtr)theEvent->message;
#endif
				break;
			
			case osEvt:
				if ((theEvent->message&osEvtMessageMask)>>24==suspendResumeMessage) {
					if (theEvent->message&1) {
						appInFront=true;
						frontWindow=FrontWindow();
						MyResumeApplication();

					} else {
						appInFront=false;
						MySuspendApplication();
					}
				}
				break;

			default:
#if Using_Carbon_API
				theDialog=GetDialogFromWindow(FrontWindow());
#else
				theDialog=FrontWindow();
#endif
				break;
		}
		
		if (MyDialogFilter(theDialog, theEvent, &mousePart) ||
				DialogSelect(theEvent, &theDialog, &mousePart))
			MyHandleDialogEvent(theEvent, theDialog, mousePart);
			
		if (theEvent->what==nullEvent)
			MyPerformIdleTasks();
			
		return;
	}

#endif

	switch (theEvent->what) {

		case nullEvent:
			MyPerformIdleTasks();
			break;
			
		case keyDown: case autoKey: // keyDown and autoKey not distinguished between
			keyCharacter=(char)(theEvent->message&charCodeMask);
			if (theEvent->modifiers&cmdKey) {
				MyEnableMenus();
				SelectMenuItem(MenuKey(keyCharacter));
			} else
				MyHandleKeyDown(keyCharacter, theEvent);
			break;
			
		case mouseDown:
			switch (mousePart) {
				case inMenuBar:
					MyEnableMenus();
					SelectMenuItem(MenuSelect(theEvent->where));
					break;
					
#if !Using_Carbon_API
				case inSysWindow:
					SystemClick(theEvent, mouseWindow);
					break;
#endif
					
				case inContent:
					if (mouseWindow==toolBarWindow) {
						realFrontWindow=frontWindow;
						frontWindow=toolBarWindow;
#if Using_Carbon_API
						SetPort(GetWindowPort(toolBarWindow));
#else
						SetPort(toolBarWindow);
#endif
						MyHandleMouseDown(localPoint, theEvent);
#if Using_Carbon_API
						SetPort(GetWindowPort(realFrontWindow));
#else
						SetPort(realFrontWindow);
#endif
						frontWindow=realFrontWindow;
					
					} else if (mouseWindow==frontWindow)
						MyHandleMouseDown(localPoint, theEvent);
					else
						SelectWindow(mouseWindow);
					break;
					
				case inDrag:
#if Using_Carbon_API
					DragWindow(mouseWindow, theEvent->where, &GetQDGlobalsScreenBits(&screenBits)->bounds);
#else
					DragWindow(mouseWindow, theEvent->where, &qd.screenBits.bounds);
#endif
					break;
					
				case inGrow:
					if (mouseWindow==frontWindow) {
						MyResizeBounds(&growLimits);
						growResult=GrowWindow(frontWindow, theEvent->where, &growLimits);

						if (growResult) {
							SizeWindow(frontWindow, LoWord(growResult), HiWord(growResult), true);
								// last parameter specifies that new bit should be updated
							MySizedWindow();
						}
					} else
						SelectWindow(mouseWindow);
					break;
					
				case inGoAway:
					if (mouseWindow==frontWindow) {
						if (TrackGoAway(frontWindow, theEvent->where)) {
							MyCloseWindow();
							frontWindow=0;
						}

					} else
						SelectWindow(mouseWindow);
					break;
					
				case inZoomIn: case inZoomOut:
					if (mouseWindow==frontWindow) {
						if (TrackBox(frontWindow, theEvent->where, mousePart)) {
							if (mousePart==inZoomOut) {
								MyZoomedSize(&zoomedWidth, &zoomedHeight);
								SetWindowStandardRect(frontWindow, zoomedWidth, zoomedHeight);
							}
							
							ZoomWindow(frontWindow, mousePart, false);
								// All sorts of things could be done here but this is default behaviour
							MySizedWindow();
						}

					} else
						SelectWindow(mouseWindow);
					break;
			}
			break;
			
		case updateEvt:
			BeginUpdate((WindowPtr)theEvent->message);
			GetPort(&oldPort);
#if Using_Carbon_API
			SetPort(GetWindowPort((WindowPtr)theEvent->message));
#else
			SetPort((WindowPtr)theEvent->message);
#endif
			MyDrawWindow((WindowPtr)theEvent->message);
			SetPort(oldPort);
			EndUpdate((WindowPtr)theEvent->message);
			break;
			
		case activateEvt:
			if (theEvent->modifiers&activeFlag) {
				frontWindow=(WindowPtr)theEvent->message;
#if Using_Carbon_API
				SetPort(GetWindowPort(frontWindow));
#else
				SetPort(frontWindow);
#endif
					// This is essential - all window-based actions depend on this call
				MyActivateWindow();

			} else {
				MyDeactivateWindow();
				frontWindow=0;
			}
			break;
			
		case osEvt:
			if ((theEvent->message&osEvtMessageMask)>>24==suspendResumeMessage) {
				if (theEvent->message&1) {
					appInFront=true;
					frontWindow=FrontWindow();
					MyResumeApplication();

				} else {
					appInFront=false;
					MySuspendApplication();
				}
			}
			break;
			
#if Use_AppleEvents
		case kHighLevelEvent:
			error=AEProcessAppleEvent(theEvent);
//			if (error!=errAEEventNotHandled)
//				TestError(error);
//			these are taken out since they could be any old scripting error
			break;
#endif

	}
}	

void SetWindowStandardRect(WindowPtr window, short zoomedWidth, short zoomedHeight)
{
//	This code adapted from DoBetterWZoom.c by Karl Groethe, Apple Computer

	Rect				windRect, zoomRect;
	Rect				globalPortRect, theSect, dGDRect;
	GDHandle			nthDevice, dominantGDevice;
	long				sectArea, greatestArea;
#if Using_Carbon_API
	BitMap				screenBits;

	GetWindowBounds(window, kWindowStructureRgn, &windRect);
#else
	windRect = (**((WindowPeek) window)->strucRgn).rgnBBox;
#endif
	dominantGDevice = nil;

	nthDevice = GetDeviceList();
	greatestArea = 0;
	while (nthDevice != nil) {
		if (TestDeviceAttribute(nthDevice, screenDevice)) {
			if (TestDeviceAttribute(nthDevice, screenActive)) {
				SectRect(&windRect, &(**nthDevice).gdRect, &theSect);
				sectArea = (long)(theSect.right-theSect.left) * (long)(theSect.bottom-theSect.top);
				if (sectArea > greatestArea) {
					greatestArea = sectArea;
					dominantGDevice = nthDevice;
				}
			}
		}
	
		nthDevice = GetNextDevice(nthDevice);
	}

	if (dominantGDevice != nil) {
		dGDRect = (**dominantGDevice).gdRect;
		if (dominantGDevice == GetMainDevice())
			dGDRect.top += GetMBarHeight() + toolBarHeight;
	
	} else {
#if Using_Carbon_API
		dGDRect = GetQDGlobalsScreenBits(&screenBits)->bounds;
#else
		dGDRect = qd.screenBits.bounds;
#endif
		dGDRect.top += GetMBarHeight() + toolBarHeight;
	}

#if Using_Carbon_API
	GetPortBounds(GetWindowPort(window), &globalPortRect);
#else
	globalPortRect = window->portRect;
#endif
	LocalToGlobal(&topLeft(globalPortRect));
	LocalToGlobal(&botRight(globalPortRect));

	dGDRect.left	+= 2 + globalPortRect.left - windRect.left;
	dGDRect.top		+= 2 + globalPortRect.top - windRect.top;
	dGDRect.right	-= 1 + windRect.right - globalPortRect.right;
	dGDRect.bottom	-= 1 + windRect.bottom - globalPortRect.bottom;

#if Using_Carbon_API
	GetWindowStandardState(window, &zoomRect);
#else
	zoomRect = (**(WStateDataHandle) ((WindowPeek) window)->dataHandle).stdState;
#endif
	
	zoomRect.left = dGDRect.left;
	zoomRect.right = (short)(zoomRect.left + zoomedWidth);
	if (zoomRect.right>dGDRect.right)
		zoomRect.right=dGDRect.right;
		
	zoomRect.top = dGDRect.top;
	zoomRect.bottom = (short)(zoomRect.top + zoomedHeight);
	if (zoomRect.bottom>dGDRect.bottom)
		zoomRect.bottom=dGDRect.bottom;
		
#if Using_Carbon_API
	SetWindowStandardState(window, &zoomRect);
#else
	(**(WStateDataHandle) ((WindowPeek) window)->dataHandle).stdState = zoomRect;
#endif
}

#pragma mark -

#if Use_AppleEvents

Error CheckRequiredParams(const AppleEvent *appleEvent)
{
	Error		error=0;
	DescType	returnType;
	Size		actualSize;

	error=AEGetAttributePtr(appleEvent, keyMissedKeywordAttr,
		typeWildCard, &returnType, 0, 0, &actualSize);

	if (error==errAEDescNotFound)
		error=0;
	else if (error==0)
		error=errAEEventNotHandled;

	return error;
}

Error InitAppleEvents()
{
	Error	error=0;
	
	// NOTE: the refCon parameter in the AEInstallEventHandler call need not be zero; you can
	// pass a long word in that will get given to your handlers when they are called

#if Handle_Open_Application_Event

	EventOpenAppUPP=NewAEEventHandlerProc(EventOpenAppHandler);
	error=AEInstallEventHandler(
		kCoreEventClass, kAEOpenApplication, EventOpenAppUPP, 0, false);
	TestError(error);
		_i(error)

#endif	

#if Handle_Open_Documents_Event

	EventOpenDocUPP=NewAEEventHandlerProc(EventOpenDocHandler);
	error=AEInstallEventHandler(
		kCoreEventClass, kAEOpenDocuments, EventOpenDocUPP, 0, false);
	TestError(error);
		_i(error)

#endif

#if Handle_Print_Documents_Event

	EventPrintDocUPP=NewAEEventHandlerProc(EventPrintDocHandler);
	error=AEInstallEventHandler(
		kCoreEventClass, kAEPrintDocuments, EventPrintDocUPP, 0, false);
	TestError(error);
		_i(error)

#endif

#if Handle_Quit_Application_Event

	EventQuitAppUPP=NewAEEventHandlerProc(EventQuitAppHandler);
	error=AEInstallEventHandler(
		kCoreEventClass, kAEQuitApplication, EventQuitAppUPP, 0, false);
	TestError(error);
	
#endif

#if Handle_Display_Notice_Event
	EventDisplayNoticeUPP=NewAEEventHandlerProc(EventDisplayNoticeHandler);
	error=AEInstallEventHandler(
		kCoreEventClass, kAESystemConfigNotice, EventDisplayNoticeUPP, 0, false);
	TestError(error);
	
#endif

	_e
	return error;
}

#if	Handle_Open_Application_Event
	
pascal OSErr EventOpenAppHandler(const AppleEvent* eventIn, AppleEvent* eventOut, unsigned long refCon)
{
	Error	error;
	
	error=CheckRequiredParams(eventIn);
		_i(error)
	
	MyOpenApplication();

	_e
	return error;
}
	
#endif

#if Handle_Open_Documents_Event

pascal OSErr EventOpenDocHandler(const AppleEvent* eventIn, AppleEvent* eventOut, unsigned long refCon)
{
	AEDescList	docList;
	DescType	returnType;
	Size		actualSize;
	long		docNum, numDocs;
	FSSpec		fileSpec;
	AEKeyword	keyword;
	Error		error=0;
	
	error=AEGetParamDesc(eventIn, keyDirectObject, typeAEList, &docList);
		_i(error)
	error=CheckRequiredParams(eventIn);
		_i(error)
	error=AECountItems(&docList, &numDocs);
		_i(error)
	
	for (docNum=1; docNum<=numDocs; docNum++) {
		error=AEGetNthPtr(&docList, docNum, typeFSS, &keyword,
			&returnType, &fileSpec, sizeof(fileSpec), &actualSize);
			_i(error)
		
		MyOpenDocument(&fileSpec);
	}
	
	error=AEDisposeDesc(&docList);

	_e
	return error;
}
	
#endif

#if Handle_Print_Documents_Event

pascal OSErr EventPrintDocHandler(const AppleEvent* eventIn, AppleEvent* eventOut, unsigned long refCon)
{
	AEDescList	docList;
	DescType	returnType;
	Size		actualSize;
	long		docNum, numDocs;
	FSSpec		fileSpec;
	AEKeyword	keyword;
	Error		error=0;
	
	error=AEGetParamDesc(eventIn, keyDirectObject, typeAEList, &docList);
		_i(error)
	error=CheckRequiredParams(eventIn);
		_i(error)
	error=AECountItems(&docList, &numDocs);
		_i(error)
	
	for (docNum=1; docNum<=numDocs; docNum++) {
		error=AEGetNthPtr(&docList, docNum, typeFSS, &keyword,
			&returnType, &fileSpec, sizeof(fileSpec), &actualSize);
			_i(error)
		
		MyPrintDocument(&fileSpec);
	}
	
	error=AEDisposeDesc(&docList);

	_e
	return error;
}
	
#endif

#if Handle_Quit_Application_Event

pascal OSErr EventQuitAppHandler(const AppleEvent* event, AppleEvent*, unsigned long)
{
	Error	error=0;
	
	error=CheckRequiredParams(event);
		_i(error);
	
	applicationHasQuit=true;

	_e
	return error;
}
	
#endif

#if Handle_Display_Notice_Event

pascal OSErr EventDisplayNoticeHandler(const AppleEvent* event, AppleEvent*, unsigned long)
{
	Error	error=0;
	
	MyHandleDisplayNotice(event);

	_e
	return error;
}
	
#endif

#endif
