/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                   */
/*        MADE - Macintosh Application Development Essentials        */
/*        ---------------------------------------------------        */
/*           (c) Sig Software, http://www.sigsoftware.com/           */
/*                                                                   */
/* These files can only be used for experimental purposes. To obtain */
/* fully commented code, source code for the functions in Essential  */
/*   Extras.h and permission for usage in final projects, you must   */
/*    purchase a license. See documentation for more information.    */
/*                                                                   */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                   */
/*  Essential Memory.c                                               */
/*  ------------------                                               */
/*                                                                   */
/*  Memory allocation, deallocation and initialisation.              */
/*                                                                   */
/*  Version 1.0.0 - 10th November 1996                               */
/*  Version 1.0.1 - 4th June 1997 - Fixed macro, function names      */
/*  Version 1.1.0 - 29th January 1998 - New OS function names        */
/*  Version 1.1.1 - 19th June 1998 - Casts for InitialiseMemory      */
/*  Version 1.2.0 - 20th November 1998 - Avoids C/C++ warnings       */
/*  Version 1.4.0 - 26th May 1999 - Added linked list routines       */
/*  Version 1.5.0 - 14th December 1999 - Added Un/RegisterPointer    */
/*                                                                   */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "Essential Headers.h"
#include "Essential Prototypes.h"

#if Project_Under_Development

typedef struct PointerRecord PointerRecord;

struct PointerRecord {
	PointerRecord		*next;
	void				*pointer;
	OSType				signature;
};

static PointerRecord	*firstPointer=0;

void RegisterPointer(void* pointer, OSType signature)
{
	PointerRecord	*record;
	
	record=(PointerRecord*)NewPtr(sizeof(PointerRecord));
	if (!record)
		Assert(false);
	record->pointer=pointer;
	record->signature=signature;
	AddToLinkedList(&firstPointer, record, firstPointer);
}

void UnregisterPointer(void* pointer, OSType signature)
{
	PointerRecord	*record;
	
	for (record=firstPointer; ; record=record->next) {
		if (!record)
			Assert(false);
		
		if ((record->pointer==pointer)&&(record->signature==signature)) {
			RemoveFromLinkedList(&firstPointer, record);
			DisposePtr((Ptr)record);
			break;
		}
	}
}

Boolean PointerExists(void* pointer, OSType signature)
{
	PointerRecord	*record;
	
	for (record=firstPointer; record; record=record->next)
		if ((record->pointer==pointer)&&(record->signature==signature))
			return true;
	
	return false;
}

#endif

void AddToLinkedList(void** firstPointer, void* addItem, void* beforeItem)
{
	void	**slotPointer;

	Assert(firstPointer);
	Assert(addItem);
	
	slotPointer=firstPointer;
	while (1) {

		if (*slotPointer==beforeItem) {
			*(void**)addItem=beforeItem;
			*slotPointer=addItem;
			break;
		}
		
		Assert(*slotPointer);
		if (*slotPointer==0)
			break;
		slotPointer=(void**)*slotPointer;
	}
}

void RemoveFromLinkedList(void** firstPointer, void* removeItem)
{
	void	**slotPointer;

	Assert(firstPointer);
	Assert(removeItem);

	slotPointer=firstPointer;
	while (1) {

		Assert(*slotPointer);
		if (*slotPointer==0)
			break;

		if (*slotPointer==removeItem) {
			*slotPointer=*(void**)removeItem;
			break;
		}
		
		slotPointer=(void**)*slotPointer;
	}
}

void* GetLinkedListItem(void* firstPointer, long itemNumber)
{
	void	*itemPointer;
	
	itemPointer=firstPointer;
	while(itemNumber) {
	
		itemPointer=*(void**)itemPointer;
		Assert(itemPointer);
		if (itemPointer==0)
			break;

		itemNumber--;
	}
	
	return itemPointer;
}

long CountLinkedListItems(void* firstPointer)
{
	void	*itemPointer;
	long	count;
	
	count=0;
	itemPointer=firstPointer;
	while (itemPointer) {
		count++;
		itemPointer=*(void**)itemPointer;
	}
	
	return count;
}

#pragma mark -

Error CreateHeapSpace(Size required)
{
	Error	error=0;
	Size	allocate;
	
	allocate=required+Emergency_Memory_Reserve;
	
	if (MaxBlock()<allocate) {
		PurgeMem(allocate);
		if (MaxBlock()<allocate) {
			MyFreeUpMemory(allocate);
			
			if (MaxBlock()<allocate)
				error=memFullErr;
		}
	}
	
	TestError(error);
	return error;
}

void*	AllocPtr(Error* error, Size allocate)
{
	void*			pointer;
	
	Assert(error);
	
	*error=CreateHeapSpace(allocate);
		_i(*error)
		
	pointer=(void*)NewPtr(allocate);
	*error=TestMemError(pointer);
		_i(*error);
		
	#if Project_Under_Development && Initialise_Allocated_Memory
		InitialiseMemory((char*)pointer, allocate);
	#endif	

	#if Project_Under_Development && Track_Memory_Allocations
		RegisterPointer(pointer, '_ptr');
	#endif	

	return pointer;
	_e
	return 0;
}

void**	AllocHandle(Error* error, Size allocate)
{
	void**	handle;
	
	Assert(error);

	*error=CreateHeapSpace(allocate);
		_i(*error)
		
	handle=(void**)NewHandle(allocate);
	*error=TestMemError(handle);
		_i(*error);
		
	#if Project_Under_Development && Move_On_Handle_Allocation
		MoveHHi((Handle)handle);
	#endif

	#if Project_Under_Development && Initialise_Allocated_Memory
		InitialiseMemory((char*)*handle, allocate);
	#endif

	#if Project_Under_Development && Track_Memory_Allocations
		RegisterPointer(pointer, '_hdl');
	#endif	

	return handle;
	_e
	return 0;
}

void DestroyPtr(void* pointer)
{
	#if Project_Under_Development && Track_Memory_Allocations
		UnregisterPointer(pointer, '_ptr');
	#endif			
			
	#if Project_Under_Development && Initialise_Allocated_Memory
		InitialiseMemory((char*)pointer, GetPtrSize((Ptr)pointer));
	#endif	

	DisposePtr((Ptr)pointer);
}

void DestroyHandle(void** handle)
{
	#if Project_Under_Development && Track_Memory_Allocations
		UnregisterPointer(pointer, '_hdl');
	#endif			
			
	#if Project_Under_Development && Initialise_Allocated_Memory
		InitialiseMemory((char*)*handle, GetHandleSize((Handle)handle));
	#endif	

	DisposeHandle((Handle)handle);
}

#if Project_Under_Development

#if Assert_Memory_Locking

// I'm sure the next two routines could be written better with more knowledge of
// Apple's System Software, but the programming guidelines don't offer such
// information, so it's likely to change. Thus I do this the long way round.

void LockHandleAssert(void** handle)
{
	SignedByte	statusBefore, statusAfter;
	
	Assert(handle);
	
	statusBefore=HGetState((Handle)handle);
	HLock((Handle)handle);
	statusAfter=HGetState((Handle)handle);
	
	Assert(statusBefore!=statusAfter);
}

void UnlockHandleAssert(void** handle)
{
	SignedByte	statusBefore, statusAfter;
	
	Assert(handle);

	statusBefore=HGetState((Handle)handle);
	HUnlock((Handle)handle);
	statusAfter=HGetState((Handle)handle);
	
	Assert(statusBefore!=statusAfter);
}
	
#endif

#if Initialise_Allocated_Memory

// This could be optimised by first filling with long words of 0xAAAAAAAA, but there's
// little point as it's only used when the project is under development anyway.

void	InitialiseMemory(char* pointer, Size length)
{
	char	*trasher, *topAddress;
	
	trasher=pointer;
	topAddress=trasher+length;
	
	while (trasher<topAddress)
		*trasher++=(char)0xAA;
}
	
#endif

#endif
