/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                   */
/*        MADE - Macintosh Application Development Essentials        */
/*        ---------------------------------------------------        */
/*           (c) Sig Software, http://www.sigsoftware.com/           */
/*                                                                   */
/* These files can only be used for experimental purposes. To obtain */
/* fully commented code, source code for the functions in Essential  */
/*   Extras.h and permission for usage in final projects, you must   */
/*    purchase a license. See documentation for more information.    */
/*                                                                   */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                   */
/*  Essential Prototypes.h                                           */
/*  ----------------------                                           */
/*                                                                   */
/*  Prototypes for functions needed mainly by MADE source files.     */
/*                                                                   */
/*  Version 1.0.0 - 10th November 1996                               */
/*  Version 1.0.1 - 4th June 1997 - Fixed macro, function names      */
/*  Version 1.1.0 - 29th January 1998 - Modeless dialog support      */
/*  Version 1.2.0 - 20th November 1998 - Open App, suspend, resume   */
/*                                       event support               */
/*  Version 1.4.0 - 26th May 1999 - Separate ProcessEvent function   */
/*  Version 1.5.0 - 14th December 1999 - Clever zooming support      */
/*  Version 1.6.0 - 22nd March 2000 - Added support for Carbon API   */
/*                                                                   */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*
	Functions you need to implement (in User Functions.c to start with)
*/

/*
		Global application control functions
*/

Error		MyInitialiseApplication();
				// Open preference files, initialise data structures, etc...
				// If this returns an error, the application will quit
void		MyClearUpApplication();
				// Close preference files, deconstruct structures (!) etc...
void		MySuspendApplication();
void		MyResumeApplication();
				// Responds to suspend and resume events e.g. activate window
void		MyPerformIdleTasks();
				// Perform any tasks when no events are forthcoming
void		MyFreeUpMemory(Size spaceNeeded);
				// Try to free memory up so spaceNeeded contiguous bytes are available. MaxBlock()
				// will tell you how you are doing. If you can't manage it, don't worry - MADE
				// handles that possibility by producing a memory full error.

/*
		Functions for specific windows
		
		NOTE: The first 8 functions (i.e. not MyDrawWindow) all operate within the context of
		the global variable frontWindow. All of them will have the current GrafPort set to
		frontWindow so you can perform any necessary drawing or other operations.
*/

void		MySetTheCursor(Point localPoint);
				// Show correct cursor for position localPoint in frontWindow; may use ShowResCursor
void		MyHandleKeyDown(char keyCharacter, EventRecord* event);
				// Deal with a key down (or auto key) event of keyCharacter, given whole event
void		MyHandleMouseDown(Point localPoint, EventRecord* event);
				// Deal with a mouse down event at localPoint, given whole event
void		MyCloseWindow();
				// Close frontWindow
void		MyActivateWindow();
				// Activate frontWindow; consider showing controls
void		MyDeactivateWindow();
				// Deactivate frontWindow; consider hiding controls
void		MyResizeBounds(Rect* growLimits);
				// Return window resizing limits for frontWindow for GrowWindow system call
void		MyZoomedSize(short* width, short* height);
				// Return window zoomed size for frontWindow (MADE chooses right screen)
void		MySizedWindow();
				// Deal with the frontWindow having been resized after SizeWindow or ZoomWindow

void		MyDrawWindow(WindowPtr drawWindow);
				// Draw the specified window. Use the visRgn to determine where it needs drawing.
				// The current GrafPort will be set correctly to drawWindow.

/*
		Menu routines
*/

void		MyEnableMenus();
				// Enable or disable all menu items as necessary; you can use EnableMenuItem. This
				// is called just before the menus are dropped down or a key equivalent used.
void		MyPerformMenu(long selection);
				// Execute the given menu item. The top 16 bits give the menu ID, and the bottom
				// 16 bits given the item number. You can use HiWord() and LoWord() to extract
				// these, or work directly on selection. External Apple Menu Items are dealt with.

/*
		Apple Event handlers
*/

#if Use_AppleEvents

#if Handle_Open_Application_Event
void		MyOpenApplication();
				// Respond to Open Application event (e.g. put up untitled window if
				// no other windows have been opened from Open Document events)
#endif

#if Handle_Open_Documents_Event
void		MyOpenDocument(FSSpec* fileSpec);
				// Respond to Open AppleEvent of specified file
#endif

#if Handle_Print_Documents_Event
void		MyPrintDocument(FSSpec* fileSpec);
				// Respond to Print AppleEvent of specified file
#endif

#if Handle_Display_Notice_Event
void		MyHandleDisplayNotice(const AppleEvent* theEvent);
				// Respond to Display Manager notification

#endif

#endif

/*
		Drag tracking and receiving
*/

#if Use_Drag_Manager

RgnHandle	MyGetDragRegion(WindowPtr inWindow, Point* localPoint, DragReference dragRef);
				// Here you must give a region for hiliting during drag tracking for
				// specified drag in inWindow at *localPoint. You must return a region that
				// can be destroyed. You can return 0 to signify no region for hiliting.
				// if localPoint is a null pointer, your window has just been left

Error		MyReceiveDrag(WindowPtr inWindow, Point localPoint, DragReference dragRef);
				// Receive the specified drag in inWindow at localPoint
				// Return an error, e.g. dragNotAcceptedError
				
#endif

/*
		Modeless dialog routines
*/

#if Support_Modeless_Dialogs

Boolean		MyDialogFilter(DialogPtr theDialog, EventRecord* event, short *itemHit);
				// Examine event, in the context of the theDialog. If the event is filtered,
				// set itemHit to the item number which was 'hit' and return true. This item
				// will be passed to MyHandleDialogEvent. Otherwise, return false. This function
				// is identical in functionality to a ModalDialog filter procedure.

void		MyHandleDialogEvent(EventRecord* event, DialogPtr theDialog, short itemHit);
				// Deal with a hit within item itemHit of the dialog theDialog, from event.

#endif

/*********************************************************************/

/*
	Functions that are implemented within the MADE package
*/

/*
		From Essential Shell.c
*/

void		ExecutionBody();

/*
		From Essential Menus.c
*/

Error		InitMenuBar();
void		SelectMenuItem(long selection);
void		SetMenuItemEnabled(short menuID, short menuItem, Boolean enable);
#if Use_Drag_Manager
Error		InitDragManager();
#endif

/*
		From Essential Events.c
*/

void		MainEventLoop();
void		ProcessEvent(EventRecord* theEvent);
#if Use_AppleEvents
Error		CheckRequiredParams(const AppleEvent* theEvent);
Error		InitAppleEvents();
#endif

/*
		From Essential Memory.c
*/

Error		CreateHeapSpace(Size required);
#if Project_Under_Development && Initialise_Allocated_Memory
void		InitialiseMemory(char* pointer, Size length);
#endif
